  program

! -------------------------------------------------------------------------------------
! Example program using the EipQManager object from Clarion 6.0/1
!
! ------------------------------------------------------------------------------------
  map
    main()
    fillQueue()
    buildStateQ()
  end

  include('abqeip.inc'),once
  include('abutil.inc'),once
  include('keycodes.clw'),once

eEipOff          equate(0)
eDefaultLineHgt  equate(9)
eDefaultAlertPos equate(255)

eNoAlertKey      equate(0)
eUpKeyPos        equate(5)
eDownKeyPos      equate(6)

eDefaultDrop     equate(8)

ePennyStep       equate(.01)

eUseCalendarYear equate(2005)
eUseSpinYear     equate(2004)

! queue to hold the some state abbr
stateQ    queue
st          string(2)
          end

! calendar to display for the date lookup, these are standard shipping calendars found in
! abutility.inc do not use the base calendar class

svCal     CalendarClass
svCal1    calendarSmallClass

! queue for the demo
testQ     queue
lName       string(20)
fName       string(15)
state       string(2)
CreditLimit decimal(7,2)
dateOrder   long
         end

!
! buffers for the eip, each field in the queue requires a local buffer for the edit process
! derive the group from the queue so there is always a buffer for each field
!
eipBuf    group(testQ)
          end

! Eip objects for each queue field
! derive the editEntryClass and overload the createControl function
!
NameEip      class(editentryclass)
CreateControl  procedure(),virtual,protected
             end

! last name and first name use the same object type 
lastNameEip   NameEip
firstNameEip  NameEip

! the creditLimit field uses a spin box
creditEip    class(editspinClass)
CreateControl  procedure(),virtual,protected
setalerts      procedure(),virtual
             end

stateEip     class(EditDropListClass)
createControl  procedure(),virtual,protected
             end

! dateOrder has two objects, Calendar class used for dates in 2005
dateOrderEip  class(EditCalendarClass)
CreateControl   procedure(),virtual,protected
              end
! dateOrder has two objects, spin class used for dates in 2004
dateSpinEip   class(EditSpinClass)
SetAlerts       procedure(),virtual,protected
              end

! field pairs class for use by the EIPManager
thisPair      FieldPairsClass

! flag for which calendar to display
useUglyCal     bool

Window WINDOW('Caption'),AT(,,395,224),FONT('MS Sans Serif',8,,FONT:regular),GRAY
       LIST,AT(21,20,297,173),USE(?eipDemoList),FORMAT('86L(2)|M~Last Name~@s20@65L(2)|M~First Name~@s15@35L(2)|M~State~@s2@50C|M~Credit' &|
           ' Limit~@n10.2@50C|M~Date Order~@d7@')
       BUTTON('&OK'),AT(309,201,35,14),USE(?OkButton),LEFT,DEFAULT
       BUTTON('&Cancel'),AT(351,201,36,14),USE(?CancelButton),LEFT
       CHECK('  Use ugly Calendar'),AT(22,204),USE(useUglyCal)
     END

! ----------------------------------------------------------------------------

! derived qEipManager, overload initcontrols method to replace object types at run time
! overload the clearColumn function to change the way the escape key works
! the other methods simply do some set up work
eipQ          class(qeipmanager)
destruct        procedure()
initControls    procedure(),virtual
clearColumn     procedure(),virtual
setupEipControl procedure(*queue q, long lFeq, *fieldPairsClass pairIn, *group buffGrp)
setupFieldPairs procedure(*group g)
setAlerts       procedure(),virtual
              end
! =============================================================================

  code

  main()

  return
! =============================================================================

main procedure()

eipResult  byte,auto

  code

  ! call set up procedures, fill the queues with something
  fillQueue()
  buildStateQ()

  open(window)

  ! set up the eip after the window is opened
  eipQ.setupEipControl(testQ, ?eipDemoList, thisPair, eipBuf)

  ! add the eip objects for each column
  eipQ.AddControl(lastNameEip, where(testq, testq.lname))
  eipQ.AddControl(firstNameEip, where(testq, testq.fname))
  eipQ.addControl(stateEip, where(testq, testq.state))
  eipQ.addControl(creditEip, where(testq, testq.creditLimit))
  ! no need to add a control to the date order field, the object is set at run time
  ! based on the year of the dateOrder
  eipQ.addControl(, where(testq, testq.dateOrder))

  ! start it
  accept
    case event()
      of event:accepted
        case field()
          ! user wants out
          of ?CancelButton
            break
        end
    of event:alertkey
      ! call the run method for the eip for the key pressed or double click on the mouse
      ! Note, the EipManager is derived from the WindowManager class, run calls Init and Kill
      case keycode()
        of mouseLeft2 orof enterKey
         eipResult = eipq.run(changeRecord)
         ! handle the result from the call to run
        of insertkey
         eipResult = eipq.run(insertRecord)
        of deletekey
         eipResult = eipq.run(deleterecord)
      end ! case keycode
    end ! case event
  end ! accept

  return
! -------------------------------------------------------------------------------------

!
! get rid of the eq instance
!
eipQ.destruct procedure()

  code

  dispose(self.eq)
  self.eq &= null

  return

! -------------------------------------------------------------------------------------
! initControls called when an eip session begins by the eipManager Init method
! used here to replace eip objects for
! the date ordered field
! dates in 2005 use a calendar lookup
!          2004 use a spin box
!          2003 can not be edited
!
! -------------------------------------------------------------------------------------
!
eipQ.initControls procedure() !virtual

  code

  ! get the eq element for the date order field
  ! test he date and set the eq control as needed
  get(self.eq, where(testq, testq.dateOrder))
  if (year(testQ.dateOrder) = eUseCalendarYear)
    ! use a calendar lookup for this year
    self.eq.control &= dateOrderEip
  elsif (year(testQ.dateOrder) = eUseSpinYear)
    ! use a spin box for this year
    self.eq.control &= dateSpinEip
  else
    ! date out of range no edits
    self.eq.control &= null
  end
  put(self.eq)

  parent.initControls()

  return
! -------------------------------------------------------------------------------------

! derived method to change unwanted action in the qEipManager
! if the key code is the esc key then do not call the parent method the method
! calls the field pairs class assignRightToLeft which is not wanted when the user
! hits the escape key
!
eipQ.clearColumn procedure() !virtual

  code

  if (keyCode() <> escKey)
    ! if not the escape key call the parent
    parent.clearColumn()
  else
    ! is the escape key close down the eip
    self.ListControl{prop:edit, self.Column} = eEipOff
    self.LastColumn = 0
  end

  return
! -------------------------------------------------------------------------------------

!
! method to perform some default set up on the eipQ object
!
eipQ.setupEipControl procedure(*queue q, long lFeq, *fieldPairsClass pairIn, *group buffGrp)

  code

  self.q &= q
  self.listcontrol = lFeq
  self.listControl{prop:from} = q
  self.listControl{prop:lineheight} = eDefaultLineHgt
  ! make a new one, no error check for this demo
  self.eq &= new(EditQueue)

  self.fields &= pairIn

  self.setupFieldPairs(buffGrp)
  self.setAlerts()

  return
! ------------------------------------------------------------------------------------

!
! set the default alert keys for the eip list control
!
eipQ.setAlerts procedure() !virtual

  code

  self.listControl{prop:alrt, eDefaultAlertPos} = insertkey
  self.listControl{prop:alrt, eDefaultAlertPos} = enterkey
  self.listControl{prop:alrt, eDefaultAlertPos} = deletekey
  self.listControl{prop:alrt, eDefaultAlertPos} = mouseLeft2

  return
! ------------------------------------------------------------------------------------

!
! init a field pairs class instance and add the fields for edit
!
! function assumes all fields in the queue have a match in the buffer, the group used
! for the buffer is derived from the queue
!
! if specific fields do not need a buffer add logic to skip the field, if a field is skipped here
! then call AddControl with the first parameter missing,  eip.addControl(, X)
! or assign the control null in the initControls function, if the control is not null
! the eip manager will use a default editEntryClass for the fields
!
eipQ.setupFieldPairs procedure(*group g)

ndx     long(1)
qf      any

  code

  self.fields.init()
  qf &= what(self.q, ndx)
  loop while (~qf &= null)
    self.fields.addPair(qf, what(g, ndx))
    ndx += 1
    qf &= what(self.q, ndx)
  end

  return
! ------------------------------------------------------------------------------------

! create control methods set up the individual controls with some defaults
!
stateEip.createControl procedure() !virtual,protected

  code

  parent.createControl()
  self.feq{prop:from} = stateq
  self.feq{prop:drop} = eDefaultDrop
  self.feq{prop:vscroll} = true

  return
! ------------------------------------------------------------------------------------

!
! call the parent and then set the cap property to true, both name fields
! will have the cap attribute set to true 
NameEip.CreateControl  procedure()

  code

  parent.createControl()
  self.feq{prop:cap} = true

  return
! ------------------------------------------------------------------------------------

creditEip.CreateControl procedure() !virtual,protected

  code

  parent.createControl()
  self.feq{prop:step} = ePennyStep
  self.feq{prop:center} = true

  return
! ------------------------------------------------------------------------------------

dateOrderEip.CreateControl procedure()  !virtual,protected

  code

  parent.createControl()
  ! set the calendar to use for the lookup can be the two shipping calendars
  ! or any calendar derived from the CalendarBaseClass
  if (useUglyCal = true)
    dateOrderEip.setCalendar(svCal1)
    ! give it a title
    self.title = 'EIP Date Demo with an Ugly Calendar'
  else
    dateOrderEip.setCalendar(svCal)
    ! give it a title
    self.title = 'EIP Date Demo'
  end

  return
! ------------------------------------------------------------------------------------

!
! the set alert methods for the spin controls call the parent and then
! remove the up and down arrow keys from the alert array for the control
! the alerts positions are hard coded in the base editClass
creditEip.setalerts procedure()

  code

  parent.setalerts()
  self.Feq{prop:alrt, eUpKeyPos} = eNoAlertKey
  self.Feq{prop:alrt, eDownKeyPos} = eNoAlertKey

  return
! ------------------------------------------------------------------------------------

dateSpinEip.setAlerts procedure() !virtual,protected

  code

  parent.setalerts()
  self.Feq{prop:alrt, eUpKeyPos} = eNoAlertKey
  self.Feq{prop:alrt, eDownKeyPos} = eNoAlertKey

  return
! ------------------------------------------------------------------------------------

! fill the queues used by the process
fillQueue procedure()

  code

  testq.lname = 'Smith'
  testq.fname = 'John'
  testq.state = 'CA'
  testq.creditLimit = 100.01
  testQ.dateOrder = date(12, 12, 2004)
  add(testq)

  testq.lname = 'Jones'
  testq.fname = 'Hank'
  testq.state = 'NY'
  testq.creditLimit = 300.57
  testQ.dateOrder = date(7, 22, 2003)
  add(testq)

  testq.lname = 'Williams'
  testq.fname = 'Frank'
  testq.state = 'FL'
  testq.creditLimit = 225.23
  testQ.dateOrder = date(2, 15, 2005)
  add(testq)

  testq.lname = 'Adams'
  testq.fname = 'Andy'
  testq.state = 'DL'
  testq.creditLimit = 223.00
  testQ.dateOrder = date(5, 13, 2003)
  add(testq)

  testq.lname = 'Barney'
  testq.fname = 'Rock'
  testq.state = 'AL'
  testq.creditLimit = 137.45
  testQ.dateOrder = date(4, 11, 2005)
  add(testq)

  return
! ------------------------------------------------------------------------------------

buildStateQ  procedure()

  code

  stateq.st = 'AL'
  add(stateQ)

  stateq.st = 'IN'
  add(stateQ)

  stateq.st = 'MN'
  add(stateQ)

  stateq.st = 'IL'
  add(stateQ)

  stateq.st = 'TN'
  add(stateQ)

  stateq.st = 'CA'
  add(stateQ)

  stateq.st = 'FL'
  add(stateQ)

  stateq.st = 'TX'
  add(stateQ)

  stateq.st = 'NY'
  add(stateQ)

  sort(stateq, +stateq.st)

  return
! ------------------------------------------------------------------------------------